module CanadaPayment::EFT::CPA005::Debit
  class Segment
    include CanadaPayment::EFT::Helper
    DEFAULT_TRANSACTION_CODE = 450

    attr_accessor :transaction_code,
                  :amount_cents,
                  :payment_date,
                  :institution,
                  :transit,
                  :account_number,
                  :client_short_name,
                  :client_long_name,
                  :client_sundry_info,
                  :client_number,
                  :operating_subsidiary,
                  :customer_name,
                  :customer_number

    def initialize(transaction_code: nil,
                   amount_cents:,
                   payment_date:,
                   institution:,
                   transit:,
                   account_number:,
                   client_short_name:,
                   client_long_name:,
                   client_sundry_info: nil,
                   client_number:,
                   operating_subsidiary: nil,
                   customer_name:,
                   customer_number:)
      @transaction_code     = transaction_code || DEFAULT_TRANSACTION_CODE
      @amount_cents         = amount_cents
      @payment_date         = payment_date
      @institution          = institution
      @transit              = transit
      @account_number       = account_number
      @client_short_name    = client_short_name
      @client_long_name     = client_long_name
      @client_sundry_info   = client_sundry_info
      @client_number        = client_number
      @operating_subsidiary = operating_subsidiary
      @customer_name        = customer_name
      @customer_number      = customer_number
    end

    def to_s
      formatted_transaction_code +
      formatted_amount +
      padded_date(@payment_date, 6) +
      zero_padded(@institution, 4) +
      zero_padded(@transit, 5) +
      space_padded(@account_number, 12) +
      "0" * 22 +
      "0" * 3 +
      space_padded(@client_short_name, 15) +
      space_padded(@customer_name, 30) +
      space_padded(@client_long_name, 30) +
      client_number_with_subsidiary +
      space_padded(@customer_number, 19) +
      "0" * 9 +
      " " * 12 +
      space_padded(@client_sundry_info, 15) +
      " " * 22 +
      " " * 2 +
      " " * 11
    end
    alias_method :output, :to_s

    private

    def formatted_transaction_code
      @transaction_code.to_s != '' ? zero_padded(@transaction_code.to_s, 3) : ' ' * 3
    end

    def formatted_amount
      raise ArgumentError.new("`amount_cents` must be an Integer") unless @amount_cents.is_a?(Integer)
      zero_padded(@amount_cents.to_s, 10)
    end

    def client_number_with_subsidiary
      ensure_length(client_number, 6) + zero_padded(operating_subsidiary, 4)
    end
  end
end
