module CanadaPayment::EFT::Helper
  def zero_padded(value, length, justify: :right)
    char_padded(value, length, "0", justify)
  end

  def space_padded(value, length, justify: :left)
    char_padded(value, length, " ", justify)
  end

  def char_padded(value, length, char, justify = :right)
    value = value.to_s
    raise ArgumentError.new("'#{value}' is longer than #{length} characters.") if value.length > length

    case justify.to_sym
    when :right
      value.rjust(length, char)
    when :left
      value.ljust(length, char)
    end
  end

  def ensure_length(value, length)
    value = value.to_s
    raise ArgumentError.new("'#{value}' should be exactly #{length} characters long.") if value.length != length
    value
  end

  def padded_date(date, length)
    zero_padded(date.year.to_s[-2..-1] + zero_padded(date.yday, 3), length)
  end
end
